( * Sensor support for 3-instruction Forth - Assad Ebrahim - 2020-07-25---Sat---21:06 )

: ndrop ( n -- ) 0 do drop loop ;


2 constant BUTTON
3 constant LED
9 constant BUZZER
13 constant SPEAKER
6  constant LASER

( * 4x4 Matrix Keypad - Membrane * )
\ TX lines, white tabbed, first four counting from left with pins facing up
4  constant tx1 
5  constant tx2 
6  constant tx3 
7  constant tx4 

\ RX lines, black tabbed, last four counting from the left with pins facing up
16  constant rx1
17  constant rx2
18  constant rx3
19  constant rx4


\ child friendly - 5 yr old Adam
: h hi ;
: l lo ;

: start-sensors 
	button pullup
	led out
	buzzer out
	speaker out
	led hi
	300 ms
	buzzer hi
	300 ms
	buzzer lo
	led lo
;

( * LASER - output * )

: laser-init
	LASER out 
	50 ms
;

: laser-on LASER hi ;
: laser-off LASER lo ;
: laser-flash 10 0 do laser-on 1500 ms laser-off 1500 ms loop ;


( * 4x4 Matrix Keypad - Membrane * )

create keypad '0' c, '1' c, '2' c, '3' c, '4' c, '5' c, '6' c, '7' c, '8' c, '9' c, 'A' c, 'B' c, 'C' c, 'D' c, '#' c, '*' c,
: ptest ( N -- ) page
	 0 do pollkeypad dup $FF = if drop else bytes keypad + c@ emit then loop ;




\  !!!!!! ONLY NEEDED DURING DEVELOPMENT OF 4x4 KEYPAD --- with readpin() -- re-use for other polled sensors

: keypad-init
	tx1 out tx2 out tx3 out tx4 out
	rx1 pullup  rx2 pullup  rx3 pullup  rx4 pullup
;


: ktest  \ matrix keypad test
\ Design: 4 wires are tx wires, 4 are rx wires.  chip needs to flip one tx wire live and poll the others to test a row/col, and then loop through the tx lines to figure out which key was pressed
	10 0 do 
	tx1 lo  tx2 hi tx3 hi tx4 hi
		rx1 readpin 1 - if ." D " then  
		rx2 readpin 1 - if ." C " then  
		rx3 readpin 1 - if ." B " then  
		rx4 readpin 1 - if ." A " then
	tx1 hi  tx2 lo tx3 hi tx4 hi
		rx1 readpin 1 - if ." # " then  
		rx2 readpin 1 - if ." 9 " then  
		rx3 readpin 1 - if ." 6 " then  
		rx4 readpin 1 - if ." 3 " then
	tx1 hi  tx2 hi tx3 lo tx4 hi
		rx1 readpin 1 - if ." 0 " then  
		rx2 readpin 1 - if ." 8 " then  
		rx3 readpin 1 - if ." 5 " then  
		rx4 readpin 1 - if ." 2 " then
	tx1 hi  tx2 hi tx3 hi tx4 lo
		rx1 readpin 1 - if ." * " then  
		rx2 readpin 1 - if ." 7 " then  
		rx3 readpin 1 - if ." 4 " then  
		rx4 readpin 1 - if ." 1 " then
		loop
;

\ CONSIDER target routine for write nibble, read nibble
: poll-keys
	tx1 low
	rx1 readpin 1 - if ." D " then  
	rx2 readpin 1 - if ." S3 " then  
	rx3 readpin 1 - if ." S2 " then  
	rx4 readpin 1 - if ." S1 " then
	tx1 high
	
	tx2 low
	rx1 readpin 1 - if ." S8 " then  
	rx2 readpin 1 - if ." S7 " then  
	rx3 readpin 1 - if ." S6 " then  
	rx4 readpin 1 - if ." S5 " then
	tx2 high
	
	tx3 low
	rx1 readpin 1 - if ." S12 " then  
	rx2 readpin 1 - if ." S11 " then  
	rx3 readpin 1 - if ." S10 " then  
	rx4 readpin 1 - if ." S9 "   then
	tx3 high
	
	tx4 low
	rx1 readpin 1 - if ." S16 " then  
	rx2 readpin 1 - if ." S15 " then  
	rx3 readpin 1 - if ." S14 " then  
	rx4 readpin 1 - if ." S13 "  then	
	tx4 high
;

: polltest 10 0 do poll-keys loop ;

