// Serial.c - AE - 2020-06-27---Sat---20:30
// compiles with: TCC

/* References:
 (1) https://www.codeproject.com/Articles/3061/Creating-a-Serial-communication-on-Win
*/


#include <windows.h>  // win32 API (includes kernel32.h and winbase.h which has serial port definitions)

// Global variables
HANDLE WINAPI hSerial;
int nErrResult=0;
unsigned char SerialBuffer[255];  // buffer for storing received data
#define ESC_KEY 0x1B  // 27d


int ErrHandling(nErrResult, err) {
    if (nErrResult==err) {
        printf("Error!  (Closing serial port ... ");
        if (CloseHandle(hSerial) == 0) { printf("Error!\n"); } else { printf("OK.) ... "); }
        exit(1);
    } // else printf("OK.\n");
}


void setupSerial(HANDLE *hSerial, char *portName, DWORD baud) {
  DCB dcbSerialParams = {0};
  COMMTIMEOUTS timeouts = {0};
        
  printf("Opening port: %s ... \n", portName);
  *hSerial = CreateFile( portName, 
        GENERIC_READ | GENERIC_WRITE,  // device mode
        0,  // not shared=0
        NULL, // default security
        OPEN_EXISTING, // file action
        0,  // default
        NULL);  // default
  ErrHandling(*hSerial, INVALID_HANDLE_VALUE);
  sleep(20); // ms
    
  printf("Device state: ... \n");
  dcbSerialParams.DCBlength = sizeof(dcbSerialParams);
  nErrResult = GetCommState(*hSerial, &dcbSerialParams);
  ErrHandling(nErrResult,0);
  sleep(20);
  
 
  printf("Port configuration: ... ");
  dcbSerialParams.BaudRate = baud; printf(" baud=%i ", baud);
  dcbSerialParams.ByteSize = 8;    printf(" bits=8 ");
  dcbSerialParams.StopBits = ONESTOPBIT;  printf(" stopbits=1 ");
  dcbSerialParams.Parity = NOPARITY;      printf(" parity=none ");
  printf("\n");
  
  nErrResult=SetCommState(*hSerial, &dcbSerialParams);
  ErrHandling(nErrResult,0);
  sleep(20);
  
  printf("COM port timeouts: ... \n");
  timeouts.ReadIntervalTimeout = 3;
  timeouts.ReadTotalTimeoutConstant = 3;
  timeouts.ReadTotalTimeoutMultiplier = 2;
  timeouts.WriteTotalTimeoutConstant = 3;
  timeouts.WriteTotalTimeoutMultiplier = 2;
    
  nErrResult=SetCommTimeouts(*hSerial, &timeouts);
  ErrHandling(nErrResult,0);
  sleep(20);

  return;
}


int sendSerial(HANDLE hSerial, char* msg, int num_bytes)
{ 
    DWORD bytes_written=0;
    
    nErrResult=WriteFile(hSerial, msg, num_bytes, &bytes_written, NULL); 
    ErrHandling(nErrResult,0);
    printf("Wrote: %d bytes\n", bytes_written);
    sleep(20);
    
    return 0;
}


int readSerial(HANDLE hSerial, unsigned char* SerialBuffer)
{
    DWORD NumBytesRead=0;
    DWORD totNumBytes=0;
    int i=0;
    
      do  // keep reading the serial port (buffer at a time) until no more characters are left
      {
        nErrResult=ReadFile( hSerial, SerialBuffer, 255, &NumBytesRead, NULL);
        ErrHandling(nErrResult,0);
        sleep(20);
        
        if (NumBytesRead>0) {
            printf("Read: ");  
            for(i=0; i<NumBytesRead; i++) { printf(" 0x%x", SerialBuffer[i]); }
            totNumBytes+=i;
        }
      } while (NumBytesRead > 0);
      if (totNumBytes >0) { printf("  Bytes: %i\n",totNumBytes); }
      return totNumBytes;
}


void waitResponse() {
    static int bytes=0;
    
    do { 
        bytes=readSerial(hSerial, SerialBuffer);
    } while (bytes==0);
    return;
}    

int main() {
    // Variables
    unsigned char msg1[6] = {1,1, 1,9, 6,0};  // pin 9 out
    unsigned char msg2[6] = {1,1, 1,9, 6,1};  // pin 9 high
    unsigned char msg3[6] = {1,0, 1,9, 6,1};  // pin 9 low
    char portName[]="COM6";
    unsigned char SerialBuffer[256]; //Circular Buffer for storing received data
    int bytes=0;
    int k=0; // index var
    unsigned char ch;   // user keypress

    
    setupSerial(&hSerial, portName, CBR_9600);  // CBR is baud rate
    waitResponse();
    
    printf("\nSending msg1: ");  sendSerial(hSerial, msg1, 6); waitResponse();
    printf("\nSending msg2: ");  sendSerial(hSerial, msg2, 6); waitResponse();
    printf("\nSending msg3: ");  sendSerial(hSerial, msg3, 6); waitResponse();
       
    printf("\nReading... (press ESC to exit)\n");
    do
    { 
      while(kbhit()==0) {  readSerial(hSerial, SerialBuffer); }  // read until key press
      ch=getch();  // getche() for echo verson
    } while (ch!=ESC_KEY) ;  // ESC to exit loop

    printf("Closing serial port...");
    if(CloseHandle(hSerial) == 0) {  printf("Error!\n"); return 1; } else printf("OK.\n");
    sleep(20);

    return 0;  // exit normally
}
